"use strict";

const _ = require('underscore'),
  EXPIRATION_IN_SECONDS = 300,
  jwt = require('jsonwebtoken'),
  Promise = require('bluebird'),
  request = require('http-as-promised'),
  url = require('url'),
  getKey = (subject, audience, payload) => `subject: ${subject} audience: ${audience} payload: ${JSON.stringify(payload)}`

class JWTGenerator {
  constructor(loginUrl, privateKey, issuer, cacheOptions) {
    cacheOptions = _.extend({}, cacheOptions, {
      max: 10,
      maxAge: Math.floor(EXPIRATION_IN_SECONDS * 1000 * 0.9),
      maxElements: 1000
    });
    this.cache = require('lru-cache')(cacheOptions);
    this.issuer = issuer;
    this.loginUrl = loginUrl;
    this.privateKey = privateKey;
  }

  make(subject, audience, payload, expiresIn) {
    const key = getKey(subject, audience, payload);
    let token = this.cache.get(key);
    if (token) {
      return Promise.resolve(token);
    }

    token = jwt.sign(
      payload || {},
      this.privateKey,
      {
        algorithm: 'RS256',
        audience: audience || 'urn:home-automation/*',
        expiresIn: expiresIn || EXPIRATION_IN_SECONDS, /* default ten minutes*/
        issuer: this.issuer,
        subject: subject
      }
    );

    return Promise
      .resolve(request({
        url: url.resolve(this.loginUrl, 'tokens'),
        method: 'POST',
        auth: {
          bearer: token
        },
        json: true,
        resolve: 'body'
      }))
      .get('token')
      .tap(this.cache.set.bind(this.cache, key));
  }

  makeNew(subject, audience, payload, expiresIn) {
    const key = getKey(subject, audience, payload);
    this.cache.del(key);
    return this.make(subject, audience, payload, expiresIn);
  }
}

module.exports = JWTGenerator;
